@@ -7,7 +7,7 @@ use warnings;
 use Data::Printer;
 use Getopt::Long::Descriptive;
 use Pod::Usage;
-use JSON;
+use JSON::MaybeXS;
 use Parse::Syslog::Line qw( parse_syslog_line );
 use YAML;
 
@@ -30,16 +30,18 @@ my ($opt,$usage) = describe_options("%c %o",
     ['Output Format'],
     ['format' => 'hidden', {
         one_of => [
-            [ print => 'Format with Data::Printer' ],
-            [ json => 'Format as JSON, minified' ],
+            [ print  => 'Format with Data::Printer' ],
+            [ json   => 'Format as JSON, minified' ],
             [ pretty => 'Format as JSON, pretty' ],
-            [ yaml => 'Format as YAML' ],
+            [ yaml   => 'Format as YAML' ],
         ],
         default => 'print',
     }],
     [],
     ['Field Control'],
-    ['sdata|s',  'Enable SDATA detection (JSON and K/V pairs)'],
+    ['disable-rfc-sdata|S', "Disables the built-in RFC5424-ish structured data extraction"],
+    ['strict-rfc-sdata|D',  "Require strict adherence to RFC5424 for structured data extraction"],
+    ['sdata|s',  'Enable all SDATA detection (JSON and K/V pairs)'],
     ['empty|e',  'Display fields with undef or blank string', ],
     ['raw|r',    'Display the raw fields in the hash.',],
     ['all|a', 'Display all fields.', { implies => [qw(raw empty)] }],
@@ -58,6 +60,8 @@ pod2usage(-verbose => 2, -exitval => 0 ) if $opt->manual;
 $Parse::Syslog::Line::PruneEmpty = !$opt->empty;
 $Parse::Syslog::Line::PruneRaw   = !$opt->raw;
 $Parse::Syslog::Line::AutoDetectJSON = $Parse::Syslog::Line::AutoDetectKeyValues = $opt->sdata;
+$Parse::Syslog::Line::RFC5424StructuredData = !$opt->disable_rfc_sdata;
+$Parse::Syslog::Line::RFC5424StructuredDataStrict = $opt->strict_rfc_sdata;
 
 while(<>) {
     my $msg = parse_syslog_line($_);
@@ -76,7 +80,7 @@ parse-syslog-line.pl - Parse a syslog message and display the structured data
 
 =head1 VERSION
 
-version 4.7
+version 5.0
 
 =head1 SYNOPSIS
 
